# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2025 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Commands related to purging of old data
"""

import os
import datetime
import shutil
import logging

import typer
from typing_extensions import Annotated

from rattail.commands import rattail_typer


log = logging.getLogger(__name__)


def run_purge(config, purge_title, purge_title_plural, thing_finder, thing_purger,
              before=None, before_days=None, default_before_days=90,
              dry_run=False, progress=None):
    from rattail.db.util import finalize_session

    log.info("will purge things of type: %s", purge_title)

    app = config.get_app()
    session = app.make_session()

    # calculate our cutoff date
    if before:
        cutoff = before
    else:
        today = app.today()
        cutoff = today - datetime.timedelta(days=before_days or default_before_days)
    cutoff = datetime.datetime.combine(cutoff, datetime.time(0))
    cutoff = app.localtime(cutoff)
    log.info("using %s as cutoff date", cutoff.date())

    # find things, and purge them
    things = thing_finder(session, cutoff, dry_run=dry_run)
    log.info("found %s thing(s) to purge", len(things or []))
    if things:
        purged = purge_things(config, session, things, thing_purger, cutoff, purge_title_plural,
                              dry_run=dry_run, progress=progress)
        log.info("%spurged %s %s",
                 "(would have) " if dry_run else "",
                 purged, purge_title_plural)

    finalize_session(session, dry_run=dry_run)


def purge_things(config, session, things, purger, cutoff, purge_title_plural,
                 dry_run=False, progress=None):
    app = config.get_app()
    result = app.make_object(purged=0)

    def purge(thing, i):
        if purger(session, thing, cutoff, dry_run=dry_run):
            result.purged += 1
        if i % 200 == 0:
            session.flush()

    app.progress_loop(purge, things, progress,
                      message=f"Purging {purge_title_plural}")
    return result.purged


@rattail_typer.command()
def purge_reports(
        ctx: typer.Context,
        before: Annotated[
            datetime.datetime,
            typer.Option(formats=['%Y-%m-%d'],
                         help="Use this date as cutoff, i.e. purge all data "
                         "*before* this date.  If not specified, will use "
                         "--before-days to calculate instead.")] = None,
        before_days: Annotated[
            int,
            typer.Option(help="Calculate the cutoff date by subtracting this "
                         "number of days from the current date, i.e. purge all "
                         "data *before* the resulting date.  Note that if you "
                         "specify --before then that date will be used instead "
                         "of calculating one from --before-days.  If neither is "
                         "specified then --before-days is used, with its default "
                         "value.")] = 90,
        dry_run: Annotated[
            bool,
            typer.Option('--dry-run',
                         help="Go through the full motions and allow logging "
                         "etc. to occur, but rollback (abort) the transaction "
                         "at the end.")] = False,
):
    """
    Purge generated reports older than a cutoff
    """
    config = ctx.parent.rattail_config
    progress = ctx.parent.rattail_progress
    app = config.get_app()
    model = app.model

    def finder(session, cutoff, dry_run=False):
        return session.query(model.ReportOutput)\
                      .filter(model.ReportOutput.created < app.make_utc(cutoff))\
                      .all()

    def purger(session, output, cutoff, dry_run=False):
        uuid = output.uuid
        log.debug("purging ReportOutput object %s: %s", uuid, output)
        session.delete(output)

        # maybe delete associated files
        if not dry_run:
            session.flush()
            key = model.ReportOutput.export_key
            path = config.export_filepath(key, uuid)
            if os.path.exists(path):
                shutil.rmtree(path)

        return True

    run_purge(config, "Generated Report", "Generated Reports",
              finder, purger,
              before=before.date() if before else None,
              before_days=before_days,
              default_before_days=90,
              dry_run=dry_run, progress=progress)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2024 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Project Generators
"""

import os
import re

from rattail.projects.base import PythonProjectGenerator


class WuttaProjectGenerator(PythonProjectGenerator):
    """
    Wutta project generator.
    """
    key = 'wutta'

    def normalize_context(self, context):
        """ """
        context = super().normalize_context(context)

        # TODO
        context['has_cli'] = True

        # classifiers
        context['classifiers'].update(set([
            'Environment :: Web Environment',
            'Framework :: Pyramid',
        ]))

        # dependencies
        context['requires'].setdefault('psycopg2', True)
        context['requires'].setdefault('WuttaWeb[continuum]', True)

        # entry point for config extension
        context['entry_points'].setdefault('wutta.config.extensions', []).extend([
            f"{context['pkg_name']} = {context['pkg_name']}.config:{context['studly_prefix']}Config"])

        # entry point for paste (web app)
        context['entry_points'].setdefault('paste.app_factory', []).extend([
            f"main = {context['pkg_name']}.web.app:main"])

        return context

    def generate_project(self, output, context, **kwargs):
        """ """
        from alembic.config import Config as AlembicConfig
        from alembic.command import revision as alembic_revision

        super().generate_project(output, context, **kwargs)

        ##############################
        # root package dir
        ##############################

        package = os.path.join(output, context['pkg_name'])

        self.generate('package/config.py.mako',
                      os.path.join(package, 'config.py'),
                      context)

        self.generate('package/commands.py.mako',
                      os.path.join(package, 'commands.py'),
                      context)

        ##############################
        # db package dir
        ##############################

        db = os.path.join(package, 'db')
        os.makedirs(db)

        self.generate('package/db/__init__.py',
                      os.path.join(db, '__init__.py'))

        ####################
        # model
        ####################

        model = os.path.join(db, 'model')
        os.makedirs(model)

        self.generate('package/db/model/__init__.py.mako',
                      os.path.join(model, '__init__.py'),
                      context)

        ####################
        # alembic
        ####################

        alembic = os.path.join(db, 'alembic')
        os.makedirs(alembic)

        versions = os.path.join(alembic, 'versions')
        os.makedirs(versions)

        # make alembic config, aware of new project versions folder
        alembic_config = AlembicConfig()
        alembic_config.set_main_option('script_location',
                                       'wuttjamaican.db:alembic')
        alembic_config.set_main_option('version_locations',
                                       ' '.join([
                                           versions,
                                           'wuttjamaican.db:alembic/versions',
                                       ]))

        # generate first revision script for new project
        script = alembic_revision(alembic_config,
                                  version_path=versions,
                                  head='wutta@head',
                                  splice=True,
                                  branch_label=context['pkg_name'],
                                  message=f"add {context['pkg_name']} branch")

        # declare `down_revision = None` ..no way to tell alembic
        # to do that apparently, so we must rewrite file
        with open(script.path, 'rt') as f:
            old_contents = f.read()
        new_contents = []
        for line in old_contents.split('\n'):
            if line.startswith('down_revision ='):
                line = re.sub(r"'\w+'", 'None', line)
            new_contents.append(line)
        with open(script.path, 'wt') as f:
            f.write('\n'.join(new_contents))

        ##############################
        # web package dir
        ##############################

        web = os.path.join(package, 'web')
        os.makedirs(web)

        self.generate('package/web/__init__.py',
                      os.path.join(web, '__init__.py'))

        self.generate('package/web/app.py.mako',
                      os.path.join(web, 'app.py'),
                      context)

        self.generate('package/web/menus.py.mako',
                      os.path.join(web, 'menus.py'),
                      context)

        self.generate('package/web/subscribers.py.mako',
                      os.path.join(web, 'subscribers.py'),
                      context)

        static = os.path.join(web, 'static')
        os.makedirs(static)

        self.generate('package/web/static/__init__.py.mako',
                      os.path.join(static, '__init__.py'),
                      context)

        libcache = os.path.join(static, 'libcache')
        os.makedirs(libcache)

        self.generate('package/web/static/libcache/README.mako',
                      os.path.join(libcache, 'README'),
                      context)

        web_templates = os.path.join(web, 'templates')
        os.makedirs(web_templates)

        self.generate('package/web/templates/base_meta.mako_tmpl',
                      os.path.join(web_templates, 'base_meta.mako'),
                      context)

        views = os.path.join(web, 'views')
        os.makedirs(views)

        self.generate('package/web/views/__init__.py.mako',
                      os.path.join(views, '__init__.py'),
                      context)
